<?php

namespace Drupal\bibleref\Plugin\Field\FieldWidget;

use Drupal\bibleref\Plugin\Field\FieldType\BibleReference;
use Drupal\bibleref\Plugin\Field\FieldType\BibleReferenceInterface;
use Drupal\clir_admin_dating_field\Plugin\Field\FieldType\ClirDatingInterface;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldWidget\OptionsWidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\OptGroup;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\taxonomy\TermStorageInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'entity_reference_autocomplete' widget.
 *
 * @FieldWidget(
 *   id = "bible_reference_widget_default",
 *   label = @Translation("Bible verse default widget"),
 *   field_types = {
 *     "bible_reference"
 *   }
 * )
 */
class BibleReferenceDefaultWidget extends OptionsWidgetBase implements ContainerFactoryPluginInterface {

  /**
   * Term storage.
   *
   * @var \Drupal\taxonomy\TermStorageInterface
   */
  protected $termStorage;

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('entity_type.manager')->getStorage('taxonomy_term')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(
    $plugin_id,
    $plugin_definition,
    FieldDefinitionInterface $field_definition,
    array $settings,
    array $third_party_settings,
    TermStorageInterface $term_storage
  ) {
    parent::__construct(
      $plugin_id,
      $plugin_definition,
      $field_definition,
      $settings,
      $third_party_settings
    );
    $this->column = BibleReference::PROPERTY_BOOK;
    $this->termStorage = $term_storage;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(
    FieldItemListInterface $items,
    $delta,
    array $element,
    array &$form,
    FormStateInterface $form_state
  ) {
    $item = isset($items[$delta]) ? $items[$delta] : NULL;
    $value = $this->getFormValues($item);

    $element[BibleReference::PROPERTY_BOOK] = parent::formElement($items, $delta, $element, $form, $form_state) + [
      '#type' => 'select',
      '#options' => $this->getOptions($items->getEntity()),
      '#default_value' => $this->getSelectedOptions($items),
      // Do not display a 'multiple' select box if there is only one option.
      '#multiple' => FALSE,
      '#required' => TRUE,
      '#weight' => 0,
    ];
    $element[BibleReference::PROPERTY_BOOK]['#title'] = $this->t('Book', [], ['context' => 'bibleref']);
    unset($element[BibleReference::PROPERTY_BOOK]['#title_display']);

    $required = !empty($element['#required']);

    $element[BibleReference::PROPERTY_CHAPTER] = [
      '#title' => $this->t('Chapter', [], ['context' => 'bibleref']),
      '#type' => 'number',
      '#required' => $required,
      '#default_value' => $value[BibleReference::PROPERTY_CHAPTER],
      '#weight' => 1,
      '#min' => 1,
    ];
    $element[BibleReference::PROPERTY_VERSE_FROM] = [
      '#title' => $this->t('Verse from', [], ['context' => 'bibleref']),
      '#type' => 'number',
      '#required' => FALSE,
      '#default_value' => $value[BibleReference::PROPERTY_VERSE_FROM],
      '#weight' => 2,
      '#size' => 3,
      '#min' => 0,
    ];
    $element[BibleReference::PROPERTY_VERSE_TO] = [
      '#title' => $this->t('to', [], ['context' => 'bibleref']),
      '#type' => 'number',
      '#required' => FALSE,
      '#default_value' => $value[BibleReference::PROPERTY_VERSE_TO],
      '#weight' => 3,
      '#size' => 3,
      '#min' => 0,
    ];
    $element[BibleReference::PROPERTY_VERSE_ADDITIONAL] = [
      '#title' => $this->t('additional', [], ['context' => 'bibleref']),
      '#type' => 'textfield',
      '#required' => FALSE,
      '#default_value' => $value[BibleReference::PROPERTY_VERSE_ADDITIONAL],
      '#weight' => 4,
      '#size' => 15,
      '#maxlength' => 50,
    ];
    $element['#theme_wrappers'][] = 'container';
    $element['#attributes']['class'][] = 'container-bibleref-widget';
    $element['#attached']['library'][] = 'bibleref/bibleref-widget';
    return $element;
  }

  protected function getFormValues(BibleReferenceInterface $item = NULL) {
    $value = [];
    if (isset($item)) {
      $array = $item->toArray();
      $value += [
        BibleReference::PROPERTY_BOOK => $array[BibleReference::PROPERTY_BOOK],
        BibleReference::PROPERTY_CHAPTER => $array[BibleReference::PROPERTY_CHAPTER],
        BibleReference::PROPERTY_VERSE_FROM => $array[BibleReference::PROPERTY_VERSE_FROM],
        BibleReference::PROPERTY_VERSE_TO => $array[BibleReference::PROPERTY_VERSE_TO],
        BibleReference::PROPERTY_VERSE_ADDITIONAL => $array[BibleReference::PROPERTY_VERSE_ADDITIONAL],
      ];
    }
    $value += [
      BibleReference::PROPERTY_BOOK => NULL,
      BibleReference::PROPERTY_CHAPTER => NULL,
      BibleReference::PROPERTY_VERSE_FROM => NULL,
      BibleReference::PROPERTY_VERSE_TO => NULL,
      BibleReference::PROPERTY_VERSE_ADDITIONAL => NULL,
    ];
    return $value;
  }

  /**
   * {@inheritdoc}
   */
  protected function getOptions(FieldableEntityInterface $entity) {
    if (!isset($this->options)) {
      /** @var \Drupal\taxonomy\TermInterface[] $terms */
      $terms = $this->termStorage->loadTree('bibleref_books', 0, 1, TRUE);
      $options = [
        '_none' => $this->t('- Select a value -'),
      ];
      foreach ($terms as $term) {
        $options[$term->id()] = $term->label();
      }

      $module_handler = \Drupal::moduleHandler();
      $context = [
        'fieldDefinition' => $this->fieldDefinition,
        'entity' => $entity,
      ];
      $module_handler->alter('options_list', $options, $context);

      array_walk_recursive($options, [$this, 'sanitizeLabel']);

      // Options might be nested ("optgroups"). If the widget does not support
      // nested options, flatten the list.
      if (!$this->supportsGroups()) {
        $options = OptGroup::flattenOptions($options);
      }

      $this->options = $options;
    }
    return $this->options;
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    $return = [];
    foreach ($values as $item) {
      if (empty($item['book'][0]['book'])) {
        continue;
      }

      $return[] = [
        BibleReference::PROPERTY_BOOK => $item['book'][0]['book'],
        BibleReference::PROPERTY_CHAPTER => $item[BibleReference::PROPERTY_CHAPTER],
        BibleReference::PROPERTY_VERSE_FROM => $item[BibleReference::PROPERTY_VERSE_FROM],
        BibleReference::PROPERTY_VERSE_TO => $item[BibleReference::PROPERTY_VERSE_TO],
        BibleReference::PROPERTY_VERSE_ADDITIONAL => $item[BibleReference::PROPERTY_VERSE_ADDITIONAL],
      ];
    }

    return $return;
  }

}
